package mi.hdm.recipes;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class RecipeSearch {
    private static final Logger log = LogManager.getLogger(RecipeSearch.class);

    private final RecipeManager recipeManager;
    private final IngredientManager ingredientManager;

    public RecipeSearch(RecipeManager recipeManager, IngredientManager ingredientManager) {
        //TODO: dont use the managers as parameters, instead just take in some List<RecipeComponent>
        this.recipeManager = recipeManager;
        this.ingredientManager = ingredientManager;
        log.debug("RecipeSearch was instantiated with {} recipes and {} ingredients", recipeManager.getRecipes().size(), ingredientManager.getAllIngredients().size());
    }

    /**
     * A method to search a list of recipes for keywords.
     * The method searches the recipe names and the ingredients for the keywords.
     *
     * @param query a String with keywords to search for
     * @return a list of Recipes which contain one or several of the keywords in its name or its ingredients
     */
    public List<Recipe> searchByQuery(List<Recipe> recipesToSearch, String query) {
        if (query == null) return recipesToSearch;
        query = query.replaceAll("[.,!+?;-]", " ").trim();
        if (query.isBlank()) return recipesToSearch;
        log.debug("Searching for {}", query);

        List<Recipe> result = new ArrayList<>();
        String[] split = query.split(" ");  //split String into separate words and put them into a String Array
        for (final Recipe r : recipesToSearch) {  //search all recipes from recipesToSearch
            for (final String s : split) {
                String l = s.toLowerCase(); //convert current string of the query to lowercase!
                if (r.getName().toLowerCase().contains(l) && !result.contains(r)) {  //check if recipe name contains words from query (convert name of recipe to lowercase too)
                    result.add(r);
                }
                for (final String key : r.getIngredients().keySet()) {
                    if (key.charAt(0) == 'i') {
                        Ingredient i = ingredientManager.getIngredient(key).orElseThrow(() -> new RuntimeException("This ingredient does not exist in ingredient manager"));
                        if (i.getName().toLowerCase().contains(l) && !result.contains(r)) result.add(r);
                    } else {
                        Recipe recipe = recipeManager.getRecipe(key).get();
                        if (recipe.getName().toLowerCase().contains(l)) result.add(recipe);
                    }
                }
            }
        }
        log.info("Search for '{}' yielded {} results.", query, result.size());
        return result;
    }

    /** A method to search a list of recipes for categories.
     * The method filters recipes which belong to all requested categories.
     *
     * @param categoriesToSearch a list of categories to which the recipes shall belong
     * @return a list of recipes which belong to all categories of categoriesToSearch
     */
    public List<Recipe> searchByCategory(List<Recipe> recipesToSearch, List<Category> categoriesToSearch) {
        if (categoriesToSearch == null || categoriesToSearch.isEmpty()) return recipesToSearch;

        List<Recipe> result = new ArrayList<>(recipesToSearch); //create List containing all the recipes to search

        for (final Recipe r : recipesToSearch) {
            for (final Category c : categoriesToSearch) {
                if (!r.getCategoryCodes().contains(c.getCode())) { //if a recipe does not contain one of the categories it will be removed from the result set
                    result.remove(r);
                }
            }
        }
        return result;
    }

    public List<RecipeComponent> searchThroughNames(String query) {
        if (query == null || query.isBlank()) {
            log.debug("Search for empty or null query yielded 0 results.");
            return List.of();
        }
        long start = System.nanoTime();
        Collection<RecipeComponent> searchList = addLists(ingredientManager.getAllIngredients().values(), recipeManager.getRecipes());
        List<RecipeComponent> result = searchList
                .stream()
                .parallel()
                .filter(c -> containsAnyToken(query, c))
                .collect(Collectors.collectingAndThen(Collectors.toList(), Collections::unmodifiableList));
        long end = System.nanoTime();
        long diffMs = (end - start) / 1_000_000; //convert from ns to ms
        log.debug("Component search for '{}' yielded {} results and took {}ms.", query, result.size(), diffMs);
        return result;
    }

    private boolean containsAnyToken(String input, RecipeComponent component) {
        String[] tokens = input.split(" ");
        for (final String token : tokens) {
            if (component.getName().toLowerCase().contains(token.toLowerCase().trim())) return true;
        }
        return false;
    }

    @SafeVarargs
    private Collection<RecipeComponent> addLists(Collection<? extends RecipeComponent>... lists) {
        List<RecipeComponent> result = new ArrayList<>();
        for (Collection<? extends RecipeComponent> l : lists) {
            if (l == null) {
                throw new IllegalArgumentException("List supplied was null");
            }
            result.addAll(l);
        }
        return result;
    }
}
