package mi.hdm.mealPlan;

import mi.hdm.exceptions.InvalidMealPlanException;
import mi.hdm.recipes.Recipe;
import mi.hdm.recipes.RecipeManager;
import mi.hdm.recipes.ValidObjectsPool;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.time.LocalDate;
import java.util.HashMap;
import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

class MealPlanTest {
    private MealPlan underTest;
    private final static RecipeManager recipeManager = ValidObjectsPool.getRecipeManager();
    private final static Recipe recipeOne = ValidObjectsPool.getValidRecipeOne();
    private final static Recipe recipeTwo = ValidObjectsPool.getValidRecipeTwo();

    @BeforeAll
    public static void setupAll() {
        recipeManager.clear();
        recipeManager.addRecipe(recipeOne);
        recipeManager.addRecipe(recipeTwo);
    }

    @BeforeEach
    public void setup() {
        underTest = new MealPlan(new HashMap<>(), recipeManager);
    }

    @Test
    public void shouldNotInitializeWithInvalidMap() {
        //given
        LocalDate date = LocalDate.now();

        assertThrows(NullPointerException.class, () -> underTest = new MealPlan(Map.of(null, recipeOne.getUniqueCode(), date, recipeTwo.getUniqueCode()), recipeManager));
    }

    @Test
    public void canAddToMealPlan() {
        //given
        LocalDate date = LocalDate.now();
        Map<LocalDate, Recipe> expected = Map.of(date, recipeOne);

        //when
        underTest.addRecipeToMealPlan(recipeOne, date);

        //expect
        assertEquals(expected, underTest.getAllRecipesFromPlan());
    }

    @Test
    public void shouldNotAddWhenDateNull() {
        assertThrows(NullPointerException.class, () -> underTest.addRecipeToMealPlan(recipeOne, null));
    }

    @Test
    public void shouldNotAddWhenOutOfBounds() {
        //given
        LocalDate future = LocalDate.of(3000, 1, 1);
        LocalDate past = LocalDate.of(2000, 1, 1);

        //expect throw
        assertThrows(InvalidMealPlanException.class, () -> underTest.addRecipeToMealPlan(recipeOne, future));
        assertThrows(InvalidMealPlanException.class, () -> underTest.addRecipeToMealPlan(recipeOne, past));
    }

    @Test
    public void shouldRemoveOldRecipes() {
        //given
        LocalDate date = LocalDate.of(2022, 1, 1);
        Map<LocalDate, String> expected = Map.of();
        HashMap<LocalDate, String> entries = new HashMap<>();
        entries.put(date, recipeOne.getUniqueCode());
        underTest = new MealPlan(entries, recipeManager);

        //when
        underTest.removePastRecipes();

        //expect
        assertEquals(expected, underTest.getAllRecipeCodesFromPlan());
    }

    @Test
    public void shouldRemoveOldRecipesButKeepUpToDate() {
        //given
        LocalDate old = LocalDate.of(2022, 1, 1);
        LocalDate now = LocalDate.now();
        HashMap<LocalDate, String> entries = new HashMap<>();
        entries.put(old, recipeOne.getUniqueCode());
        entries.put(now, recipeTwo.getUniqueCode());
        underTest = new MealPlan(entries, recipeManager);

        Map<LocalDate, Recipe> expected = Map.of(now, recipeTwo);

        //when
        underTest.removePastRecipes();

        //expect
        assertEquals(expected, underTest.getAllRecipesFromPlan());
    }

    @Test
    public void shouldClearDayIfPresent() {
        //given
        Map<LocalDate, Recipe> expected = Map.of();
        LocalDate date = LocalDate.now();
        underTest.addRecipeToMealPlan(recipeOne, date);

        //when
        underTest.clear(date);

        //expect
        assertEquals(expected, underTest.getAllRecipesFromPlan());
    }
}
